#include <SPI.h>
#include <ArduIoT_U1.h>
#include <ArduIoT_RTC.h>
#include <Wire.h>
#include <EEPROM.h>

ArduIoT_U1 lcd(A0, A1);
aIoTMenu menu;
ArduIoT_RTC rtc;

void VOID() { }

MENU menuOptions[] = {
  {0, "     M e n u", 't', VOID, 0},
  {0, "Temp on Base", 'f', baseTf, 0},         //'t' for menu Title
  {0, "Base Active Time", 'f', baseActiveAfterF, 0},    //'f' is for a Menu Function. If choisen runs HelloLoop function
  {0, "Base Beep Time ", 'f', baseBeepAfterF, 0},
  {0, "Base Standby Time", 'f', baseStandbyAfterF, 0},
  {0, "Increment Step", 'f', stepTF, 0},
  {0, "Min Set Temp", 'f', setTminF, 0},
  {0, "Max Set Temp", 'f', setTmaxF, 0},
  {0, "Cooling OFF temp", 'f', coolingTF, 0},
  {0, "Delta Minus Value", 'f', deltaMinusF, 0},
  {0, "Delta Plus  Value", 'f', deltaPlusF, 0},
  {0, "On Power On/Standby", 'f', standByModeF, 0},
  {0, "Not Connected Value", 'f', notConnectedTF, 0},
  {0, "Hour Set   ", 'f', clockSetHour, 0},
  {0, "Minute Set ", 'f', clockSetMinute, 0},
  {0, "Second Set ", 'f', clockSetSecond, 0},
  {0, "Factory Reset", 'f', factoryResetF, 0},

  {255, "", ' ', 0},                        //Every menu array must end like that.
};

byte ver=19;


unsigned int baseActiveAfter = 60; //activate base on after x sec
unsigned int baseBeepAfter = 1740;
unsigned int baseStandbyAfter = 1800;
int baseT = 200;
int setT = 260;
byte stepT = 10;
int setTmin = 100;
int setTmax = 440;
byte deltaMinus = 1;
byte deltaPlus = 3;
int coolingT = 150;
byte standByMode = HIGH;
int notConnectedT = 540;


int ironPin = 3;
int tempPin = A3;
int basePin = 2;

byte baseSensor = HIGH;
byte prevbaseSensor = 3; //first time is always  0 or 1
byte baseOf = LOW; //high when takeing soldering of base until hits setT again
byte baseState = 0;

unsigned int baseStartTime = 0;

byte heater = LOW;
unsigned long int lastRotaryChange = 0;
unsigned long int lastRefresh = 0;

int targetT;

int currentT = 0;
int prevCurrentT = 1;
byte notConnectedDisplay = LOW;


void setup() {
  Wire.begin();
  pinMode(ironPin, OUTPUT);
  pinMode(basePin, INPUT);
  lcd.createCustomChars(); //and clears screen
  rtc.init();
  displayWelcome();
  if  (checkEEPROM()) {
    lcd.clear();
    lcd.printCursor(2, 2, "Loading Setting");
    delay(1000);
    readEEPROM();
  }
  else {
    lcd.clear();
    lcd.printCursor(4, 1, "FACTORY RESET");
    lcd.printCursor(7, 3, " DONE.");
    delay(3000);
    saveEEPROM();
  }
  targetT = setT;
  lcd.clear();
}



void loop() {
  int rotaryTurnStatus = lcd.rotary();
  byte switchTop = lcd.swTop();
  byte switchBottom = lcd.swBottom();
  byte switchRotary = lcd.swRotary();


  if (standByMode == HIGH) {
    turnOff();
    lcd.led(4);

    lcd.setBackLight(LOW);
    if ((rotaryTurnStatus != 0) || (switchTop != 0) || (switchBottom != 0)) displayMessageOnStandBy();
    if (switchRotary == SW_RELEASE) waikupMode();
  }

  else {         //its working!!!! not standBy

    readTemp();  //read sensor Temp. if its high, its not connected.
    if (currentT > notConnectedT) {  //check if soldering iron is connected
      turnOff();
      notConnectedDisplay = HIGH;
      displayNotConnected();
    }

    else {         //ok. its working and its connected. lets run..
      checkTemp();  //check temp. and compare it with 'delta' variables to turn it on or off
      checkBase();  //if is on base then do some acts
      displayEvery1sec();  //display info to LCD every 1 second

      if (notConnectedDisplay == HIGH) {  //if for some reason you unplug the soldering, then display that
        notConnectedDisplay = LOW;
        checkTemp();
        displayInfo();
      }

      if (switchBottom == SW_RELEASE) runStandByMode();  //if bottom button is press, then go to standby mode
      if (switchTop == SW_RELEASE) runMenuLoop();

      //lets change temp. from rotary.

      if (rotaryTurnStatus == ROTARY_FORWARD) {
        if (switchRotary == SW_PRESS)
          setT = setT + 100;
        else
          setT = setT + stepT;
        if (setT > setTmax) setT = setTmax;
        lastRotaryChange = millis();
        baseState = 0;                  //if on base reset status and timers
        baseStartTime = millis() / 1000;
        targetT = setT;
        displaysetT();
      }

      if (rotaryTurnStatus == ROTARY_BACKWARD) {
        if (switchRotary == SW_PRESS)
          setT = setT - 100;
        else
          setT = setT - stepT;
        if (setT < setTmin) setT = setTmin;
        lastRotaryChange = millis();
        baseState = 0;                   //if on base reset status and timers
        baseStartTime = millis() / 1000;
        targetT = setT;
        displaysetT();
      }
    }
  }
}

void checkBase() {
  baseSensor = digitalRead(basePin);

  if (prevbaseSensor != baseSensor) {  //there is a change on base
    if (prevbaseSensor == LOW)     {// change from base on to base off
      baseOf = HIGH;
      baseStartTime = 0;
      baseState = 0;
      targetT = setT;
    }
    if (prevbaseSensor == HIGH) { //just got on base
      baseStartTime = millis() / 1000;
      baseState = 0;
    }
    prevbaseSensor = baseSensor;
    displayInfo();
  }

  if (baseSensor == HIGH) {  //of base !!!!
    targetT = setT;
  }
  else {  //    on base !!!!!!
    if (((baseStartTime + baseActiveAfter) < (millis() / 1000)) && (baseState == 0)) {
      baseState = 1;
      buzzFor(5);
      if (baseT < setT) targetT = baseT;
    }
    if (((baseStartTime + baseBeepAfter) < (millis() / 1000)) && (baseState == 1)) {
      buzzFor(100);
      baseState = 2;
    }
    if (((baseStartTime + baseStandbyAfter) < (millis() / 1000)) && (baseState == 2)) {
      buzzFor(200);
      baseState = 0;
      runStandByMode() ;
    }
  }
}

void readTemp() {
  currentT = analogRead(A3) / 1.3;
}

void checkTemp() {
  readTemp();
  if (((currentT + deltaPlus) < targetT) && (heater == LOW)) {
    turnOn();
  }
  if (((currentT - deltaMinus) > targetT) && (heater == HIGH)) {
    turnOff();
    if ((baseOf == HIGH) && (baseSensor == HIGH)) { // it was on base and now is off. first time switching off after pickup from base
      baseOf = LOW;
      buzzFor(5);
    }
  }
}


void turnOff() {
  digitalWrite(ironPin, LOW);
  lcd.led(0);
  heater = LOW;
}
void turnOn() {
  digitalWrite(ironPin, HIGH);
  lcd.led(1);
  // buzzFor(1);
  heater = HIGH;
}

void buzzFor(int x) {
  lcd.setBuzzer(HIGH);
  delay(x);
  lcd.setBuzzer(LOW);
}

void displaysetT() {
  lcd.clear();
  if ((lastRotaryChange < (millis() + 1000)) && (lastRotaryChange != 0)) {
    lastRotaryChange = 0;
    lastRefresh = millis();
    lcd.printBig(0, 2, String(setT));
    lcd.printCursor(12, 3, "C");
  }
}

void displayEvery1sec() {
  if (lastRefresh + 1000 < (millis())) {
    lastRefresh = millis();
    displayInfo();
  }
}

void displayInfo() {
  lcd.clear();
  if (standByMode == LOW) {
    lcd.printCursor(15, 0, rtc.getTime24HHMM());
    lcd.printCursor(0, 0, "Current ");
    lcd.printCursor(8, 0, String(currentT) + " C");
    if (baseSensor == LOW) {                        //if is on base
      lcd.printBig(2, 1, "BASE");
      lcd.printCursor(0, 3, "Target " + String(targetT) + "C ");
      lcd.print(millis() / 1000 - baseStartTime);
      lcd.print("sec");
    }
    else {
      lcd.printBig(4, 2, String(setT));
      lcd.printCursor(16, 3, "C");
    }
  }
}

void displayNotConnected() {
  lcd.clear();
  lcd.printCursor(3, 0, "Soldering Iron");
  lcd.printCursor(2, 2, "is NOT Connected");
  lcd.printCursor(4, 3, "OR Very Hot!");
  lcd.printCursor(16, 0, currentT);
  
  delay(100);
}

void displayWelcome() {
  lcd.printCursor(3, 0, "arduIoT.com");
  lcd.printCursor(0, 2, "Soldering Iron Demo");
  lcd.printCursor(15,3,"v0."+String(ver));
  delay(2000);
}

void waikupMode() {
  lcd.clear();
  lcd.printCursor(3, 2, "Waiking Up");
  standByMode = LOW;
  lcd.setDim(HIGH);
  lcd.led(0);
  baseStartTime = millis() / 1000;
  prevbaseSensor = 3; //never true
  baseState = 0;
}

void runStandByMode() {
  byte changeMindAndWaikup = LOW;
  turnOff();
  lcd.clear();
  lcd.printBig(0, 0, "Cool..");
  lcd.printCursor(0, 3, "Cancel-Press Rotary");
  readTemp();
  byte i = 0;
  while ((currentT > coolingT) && (changeMindAndWaikup == LOW)) {
    readTemp();
    if ( lcd.swRotary() == SW_RELEASE) changeMindAndWaikup = HIGH;
    lcd.printCursor(7, 2, String(currentT) + " C");
    lcd.led(i = !i);
    delay(100);
  }
  if (changeMindAndWaikup == LOW) {
    lcd.clear();
    lcd.printCursor(8, 2, "BYE!");
    standByMode = HIGH;
    delay(1000);
    lcd.setDim(LOW);
    lcd.clear();
  }
  else {
    waikupMode();
  }
  lcd.clear();
}

void  displayMessageOnStandBy() {
  lcd.clear();
  lcd.printCursor(3, 2, "Press Rotary");
  lcd.setBackLight(HIGH);
  delay(500);
  lcd.setBackLight(LOW);
}


/*
  byte standByMode = LOW;
  byte deltaMinus = 3;
  byte deltaPlus = 3;
  byte stepT = 10;

  int baseT = 200;
  int setT = 260;
  int setTmin = 100;
  int setTmax = 480;
  int coolingT = 150;
  int notConnectedT = 503;

  unsigned int baseActiveAfter = 30; //activate base on after x sec
  unsigned int baseBeepAfter = 1800;
  unsigned int baseStandbyAfter = 1860;

*/
//****************************************************     EEPROM functions
void readEEPROM() {
  standByMode = read8(102);
  deltaMinus = read8(103);
  deltaPlus = read8(104);
  stepT = read8(105);

  baseT = read16(110);
  setT = read16(112);
  setTmin = read16(114);
  setTmax = read16(116);
  coolingT = read16(118);
  notConnectedT = read16(120);
  baseActiveAfter = read16(122);
  baseBeepAfter = read16(124);
  baseStandbyAfter = read16(126);
}

void saveEEPROM() {
  lcd.clear();
  lcd.print("Save to EEPROM");
  delay(800);
  write8(100, 111);
  write8(101, 132);
  write8(102, standByMode);
  write8(103, deltaMinus);
  write8(104, deltaPlus);
  write8(105, stepT);

  write16(110, baseT);
  write16(112, setT);
  write16(114, setTmin);
  write16(116, setTmax);
  write16(118, coolingT);
  write16(120, notConnectedT);
  write16(122, baseActiveAfter);
  write16(124, baseBeepAfter);
  write16(126, baseStandbyAfter);
}

byte checkEEPROM() {
  if ((read8(100) == 111) && (read8(101) == 132))
    return HIGH;
  else
    return LOW;
}


void write16( int a, unsigned int i) {
  byte i2 = i / 256;
  byte i1 = i - i2 * 256;
  EEPROM.write(a, i1);
  EEPROM.write(a + 1, i2);
}
void write8(int a, byte i)  {
  EEPROM.write(a, i);
}
byte read8(int a) {
  return EEPROM.read(a);
}
unsigned int read16(int a) {
  byte i1, i2;
  i1 = EEPROM.read(a);
  i2 = EEPROM.read(a + 1);
  return i1 + i2 * 256;
}




/****************************************** MENU FUNCTIONS
  MENU menuOptions[] = {
  {0, "     M e n u", 't', VOID, 0},
  {0, "Temp on Base", 'f', baseTf, 0},         //'t' for menu Title
  {0, "Base Active Time", 'f', baseActiveAfterF, 0},    //'f' is for a Menu Function. If choisen runs HelloLoop function
  {0, "Base Beep Time ", 'f', baseBeepAfterF, 0},
  {0, "Base Standby Time", 'f', baseStandbyAfterF, 0},
  {0, "Increment Step", 'f', stepTF, 0},
  {0, "Min Set Temp", 'f', setTminF, 0},
  {0, "Max Set Temp", 'f', setTmaxF, 0},
  {0, "Cooling OFF temp", 'f', coolingTF, 0},
  {0, "Delta Minus Value", 'f', deltaMinusF, 0},
  {0, "Delta Plus  Value", 'f', deltaPlusF, 0},
  {0, "On Power On/Off", 'f', standByModeF, 0},
  {0, "Not Connected Value Set", 'f', notConnectedTF, 0},
  {0, "Hour Set   ", 'f', clockSetHour, 0},
  {0, "Minute Set ", 'f', clockSetMinute, 0},
  {0, "Second Set ", 'f', clockSetSecond, 0},
  {0, "Factory Reset", 'f', factoryResetF, 0},

  {255, "", ' ', 0},                        //Every menu array must end like that.
  };
*/


void runMenuLoop() {
  turnOff();
  menu.menu = menuOptions;
  menu.init('>', sizeof(menuOptions), lcd);
  byte switchTop = lcd.swTop();
  byte switchBottom = lcd.swBottom();
  while ((switchTop != SW_RELEASE) && (switchBottom != SW_RELEASE)) {
    switchTop = lcd.swTop();
    switchBottom = lcd.swBottom();
    menu.runMenu();
  }
}




void baseTf() {
  baseT = getValue("Base Standby Temp",  100, 300, baseT, 10);
  saveEEPROM();
}
void baseActiveAfterF() {
  baseActiveAfter = getValue("Base Active after", 0, 150, baseActiveAfter, 5);
  saveEEPROM();
}
void baseBeepAfterF() {
  baseBeepAfter = getValue  ("Beep Before Standby", baseActiveAfter + 5, 12000, baseBeepAfter, 60);
  saveEEPROM();
}
void baseStandbyAfterF() {
  baseStandbyAfter = getValue("StandBy after Time", baseBeepAfter, 12360, baseStandbyAfter, 60);
  saveEEPROM();
}
void stepTF() {
  stepT = getValue("Step up/down value", 1, 50, stepT, 1);
  saveEEPROM();
}
void setTminF() {
  setTmin = getValue("Minimum Temp Set", 0, 300, setTmin, 10);
  saveEEPROM();
}
void setTmaxF() {
  setTmax = getValue("Maximum Temp Set", 200, 480, setTmax, 10);
  saveEEPROM();
}
void coolingTF() {
  coolingT = getValue("Cooling Temp", 50, 300, coolingT, 10);
  saveEEPROM();
}
void deltaMinusF() {
  deltaMinus = getValue("Delta Temp Minus", 0, 20, deltaMinus, 1);
  saveEEPROM();
}
void deltaPlusF() {
  deltaPlus = getValue("Delta Temp Plus", 0, 20, deltaPlus, 1);
  saveEEPROM();
}
void standByModeF() {
  standByMode = !getValue("0-standby, 1-on", 0, 1, 0, 1);
  saveEEPROM();
}
void notConnectedTF() {
  notConnectedT = getValue("Not Connected Value", 450, 600, notConnectedT, 1);
  saveEEPROM();
}

void clockSetSecond() {
  rtc.setSecond(getValue(F("Set Second"), 0, 59, rtc.getSecond(), 1));
}

void clockSetMinute() {
  rtc.setMinute(getValue(F("Set Minutes"), 0, 59, rtc.getMinute(), 1));
}
void clockSetHour() {
  rtc.setHour(getValue(F("Set Hour"), 0, 23, rtc.getHour(), 1));
}

void factoryResetF() {
  write8(100, 0);
  write8(101, 0);
  lcd.clear();
  lcd.printCursor(4, 1, "Press Reset on");
  lcd.printCursor(2, 3, "Soldering Station");
  while (1) {}
}


int getValue(String st,  int iMin, int iMax, int iStart, int stepi) {
  lcd.clear();
  lcd.print(st);
  lcd.printCursor(0, 1, F("Between ")); lcd.print(iMin); lcd.print(" - "); lcd.print(iMax);
  byte swRotary = 0;
  byte swBottom = 0;
  byte rotary = 0;
  int i = iStart;
  lcd.setCursor(5, 3);
  lcd.print(i);
  while ((swRotary != 2) && (swBottom < 2)) {
    swRotary = lcd.swRotary();
    swBottom = lcd.swBottom();
    rotary = lcd.rotary();
    byte change = LOW;

    if (swBottom == 2) i = iStart;
    if (rotary == ROTARY_FORWARD) {
      change = HIGH;
      if (swRotary == SW_PRESS) {
        swRotary = 0;
        i = i + stepi * 10;          //press rotary to go 10x faster!!!!!!!!!!!!!!!
      }
      else
        i = i + stepi ;
    }
    if (rotary == ROTARY_BACKWARD) {
      change = HIGH;
      if (swRotary == SW_PRESS) {
        swRotary = 0;
        i = i - stepi * 10;
      }
      else
        i = i - stepi ;
    }
    if (i < iMin) i = iMin;
    if (i > iMax) i = iMax;
    if (change) {
      lcd.printCursor(5, 3, "         ");
      lcd.setCursor(5, 3);
      lcd.print(i);
    }
  }
  lcd.clear();
  lcd.print(st);
  lcd.printCursor(0, 2, "Value Is  : " + String(i));
  delay(800);
  return i;
}


